@@ -1,17 +1,17 @@
 #' Variable Filter
 #'
-#' This function filter variables base on the specified conditions, including minimum iv, maximum na percentage and maximum element percentage.
+#' This function filter variables base on specified conditions, such as information value, missing rate, identical value rate.
 #'
 #' @param dt A data frame with both x (predictor/feature) and y (response/label) variables.
 #' @param y Name of y variable.
-#' @param x Name of x variables. Default NULL If x is NULL, all variables exclude y will counted as x variables.
-#' @param iv_limit The minimum IV of each kept variable, default 0.02.
-#' @param na_perc_limit The maximum rate of NAs in each kept variable, default 0.95.
-#' @param ele_perc_limit The maximum rate of identical value (excluding NAs) in each kept variable, default 0.95.
-#' @param var_rm Name vector of force removed variables, default NULL.
-#' @param var_kp Name vector of force kept variables, default NULL.
-#' @param return_rm_reason Logical, default FALSE.
-#' @param positive Value of positive class, default "bad|1".
+#' @param x Name of x variables. Default is NULL. If x is NULL, then all variables except y are counted as x variables.
+#' @param iv_limit The information value of kept variables should >= iv_limit. The default is 0.02.
+#' @param missing_limit The missing rate of kept variables should <= missing_limit. The default is 0.95.
+#' @param identical_limit The identical value rate (excluding NAs) of kept variables should <= identical_limit. The default is 0.95.
+#' @param var_rm Name of force removed variables, default is NULL.
+#' @param var_kp Name of force kept variables, default is NULL.
+#' @param return_rm_reason Logical, default is FALSE.
+#' @param positive Value of positive class, default is "bad|1".
 #' @return A data.table with y and selected x variables and a data.table with the reason of removed x variable if return_rm_reason == TRUE.
 #'
 #' @examples
@@ -25,7 +25,7 @@
 #' @import data.table
 #' @export
 #'
-var_filter = function(dt, y, x = NULL, iv_limit = 0.02, na_perc_limit = 0.95, ele_perc_limit = 0.95, var_rm = NULL, var_kp = NULL, return_rm_reason = FALSE, positive="bad|1") {
+var_filter = function(dt, y, x = NULL, iv_limit = 0.02, missing_limit = 0.95, identical_limit = 0.95, var_rm = NULL, var_kp = NULL, return_rm_reason = FALSE, positive="bad|1") {
   . = info_value = variable = rt = rm_reason = NULL # no visible binding for global variable
 
   # set dt as data.table
@@ -47,7 +47,7 @@ var_filter = function(dt, y, x = NULL, iv_limit = 0.02, na_perc_limit = 0.95, el
   # -na percentage
   na_perc = dt[, sapply(.SD, function(a) sum(is.na(a))/length(a)), .SDcols = x]
   # -element percentage
-  ele_perc = dt[, sapply(.SD, function(a) max(table(a)/sum(!is.na(a)))), .SDcols = x]
+  ele_perc = dt[, sapply(.SD, function(a) max(table(a))/sum(!is.na(a)) ), .SDcols = x]
 
   # datatable  iv na ele
   dt_var_selector =
@@ -56,7 +56,7 @@ var_filter = function(dt, y, x = NULL, iv_limit = 0.02, na_perc_limit = 0.95, el
 
   # remove na_perc>95 | ele_perc>0.95 | iv<0.02
   # variable datatable selected
-  dt_var_sel = dt_var_selector[info_value >= iv_limit & na_perc <= na_perc_limit & ele_perc <= ele_perc_limit]
+  dt_var_sel = dt_var_selector[info_value >= iv_limit & na_perc <= missing_limit & ele_perc <= identical_limit]
 
   # add kept variable
   x_selected = dt_var_sel[, as.character(variable)]
@@ -66,12 +66,12 @@ var_filter = function(dt, y, x = NULL, iv_limit = 0.02, na_perc_limit = 0.95, el
   if (return_rm_reason) {
     # variable datatable deleted
     dt_var_rm = dt_var_selector[
-      info_value < iv_limit | na_perc > na_perc_limit | ele_perc > ele_perc_limit
+      info_value < iv_limit | na_perc > missing_limit | ele_perc > identical_limit
     ][, `:=`(
       info_value = ifelse(info_value < iv_limit, paste0("iv < ", iv_limit), ""),
-      na_perc = ifelse(na_perc > na_perc_limit, paste0("miss rate > ",na_perc_limit), ""),
-      ele_perc = ifelse(ele_perc > ele_perc_limit, paste0("identical rate > ", ele_perc_limit), "")
-    )][]
+      na_perc = ifelse(na_perc > missing_limit, paste0("miss rate > ",missing_limit), ""),
+      ele_perc = ifelse(ele_perc > identical_limit, paste0("identical rate > ", identical_limit), "")
+    )]
 
     dt_rm_reason = melt(
       dt_var_rm, id.vars = "variable", variable.name="var", value.name="rm_reason", variable.factor=TRUE
